import { type ValidationContext } from '../../validation/validation-context';
import { type BinaryExpression, type TernaryExpression, type UnaryExpression } from '../generated/ast';
import { type ValueType } from '../wrappers/value-type/value-type';
export interface UnaryOperatorTypeComputer {
    /**
     * Computes the type of a unary operator by the type of its operand.
     * @param operandType the type of the operand
     * @param expression the expression to use for diagnostics
     * @param context the validation context to use for diagnostics
     * @returns the resulting type of the operator or `undefined` if the type could not be inferred
     */
    computeType(operandType: ValueType, expression: UnaryExpression, context: ValidationContext | undefined): ValueType | undefined;
}
export declare abstract class DefaultUnaryOperatorTypeComputer implements UnaryOperatorTypeComputer {
    protected readonly expectedOperandType: ValueType;
    constructor(expectedOperandType: ValueType);
    computeType(operandType: ValueType, expression: UnaryExpression, context: ValidationContext | undefined): ValueType | undefined;
    protected abstract doComputeType(operandType: ValueType): ValueType;
}
export interface BinaryOperatorTypeComputer {
    /**
     * Computes the type of a binary operator by the type of its operands.
     * @param leftType the type of the left operand
     * @param rightType the type of the right operand
     * @param expression the expression to use for diagnostics
     * @param context the validation context to use for diagnostics
     * @returns the resulting type of the operator or `undefined` if the type could not be inferred
     */
    computeType(leftType: ValueType, rightType: ValueType, expression: BinaryExpression, context: ValidationContext | undefined): ValueType | undefined;
}
export declare abstract class DefaultBinaryOperatorTypeComputer implements BinaryOperatorTypeComputer {
    protected readonly expectedLeftOperandType: ValueType;
    protected readonly expectedRightOperandType: ValueType;
    constructor(expectedLeftOperandType: ValueType, expectedRightOperandType: ValueType);
    computeType(leftOperandType: ValueType, rightOperandType: ValueType, expression: BinaryExpression, context: ValidationContext | undefined): ValueType | undefined;
    protected abstract doComputeType(leftOperandType: ValueType, rightOperandType: ValueType): ValueType;
}
export declare function generateUnexpectedTypeMessage(expectedType: ValueType, actualType: ValueType): string;
export declare abstract class DefaultTernaryOperatorTypeComputer implements TernaryOperatorTypeComputer {
    protected readonly expectedFirstOperandType: ValueType;
    protected readonly expectedSecondOperandType: ValueType;
    protected readonly expectedThirdOperandType: ValueType;
    constructor(expectedFirstOperandType: ValueType, expectedSecondOperandType: ValueType, expectedThirdOperandType: ValueType);
    computeType(firstOperandType: ValueType, secondOperandType: ValueType, thirdOperandType: ValueType, expression: TernaryExpression, context: ValidationContext | undefined): ValueType | undefined;
    protected abstract doComputeType(firstOperandType: ValueType, secondOperandType: ValueType, thirdOperandType: ValueType): ValueType;
}
export interface TernaryOperatorTypeComputer {
    /**
     * Computes the type of a ternary operator by the type of its operands.
     * @param firstType the type of the first operand
     * @param secondType the type of the second operand
     * @param thirdType the type of the third operand
     * @param expression the expression to use for diagnostics
     * @param context the validation context to use for diagnostics
     * @returns the resulting type of the operator or `undefined` if the type could not be inferred
     */
    computeType(firstType: ValueType, secondType: ValueType, thirdType: ValueType, expression: TernaryExpression, context: ValidationContext | undefined): ValueType | undefined;
}
